#include "raven_spn.h"
#include "spn_regs.h"
#include "init.h"

using spn =spn_regs<0x90000000>;

// huge arrays of XSPN input and referance data
extern std::array<uint8_t, 50000> input_data;
extern std::array<double, 10000> ref_data;

bool double_equals(double a, double b, double epsilon = 0.001)
{
    return std::abs(a - b) < epsilon;
}

void run_xspn(int in_addr, int out_addr) {
	spn::mode_reg() = 0;
	spn::input_length_reg() = 500; 		// each sample consists of 5 uint8 values
	spn::input_addr_reg() = in_addr;
	spn::output_addr_reg() = out_addr;
	spn::num_of_in_beats_reg() = 40;	// Number of AXI4 burst beats needed to load all input data
	spn::num_of_out_beats_reg() = 64;	// Number of AXI4 burst beats needed to store all result data
	spn::start_reg() = 1;
}

void check_results(int addr, int k) {
	bool result = 0;
	int  step = 500;
	double *res_base = (double*) (addr);
	int * error_exit = (int *)0xF0000000;
    printf("Start result comparison %d - %d\n", k, k+step);

	for (int i = 0; i < step; i++) {
		if (!double_equals(res_base[i], ref_data.at(k + i))) {
			printf("XSPN ref %d comparison FAILED\n", k + i);
			*error_exit = 0x1;
			result = 1;
		}
	}
    printf("Compared samples %d - %d with the reference\n", k, k+step);
}

/*! \brief main function
 *
 */
int main() {
    platform_init();

    spn::mode_reg() = 1;
    spn::start_reg() = 1;
    wait_for_interrupt();
    spn::interrupt_reg() = 1;
    printf("READOUT HW:0x%x\n", spn::readout_reg());

	int in_addr  = (int)input_data.data();
	int out_addr = 0x800B0000;

    run_xspn(in_addr, out_addr);

    int step = 500; // number of samples to be process at once
    for (int k = 0; k < 10000; k+=step) {
        printf("XSPN processes samples %d - %d\n", k, k+step);

        wait_for_interrupt();
    	spn::interrupt_reg() = 1;
		check_results(out_addr, k);

		in_addr += step * 5; // 5 bytes in each sample
        run_xspn(in_addr, out_addr);
	}

	return 0;
}
